"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const Rx = __importStar(require("rxjs"));
const operators_1 = require("rxjs/operators");
const webSocket_1 = require("rxjs/webSocket");
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const types_1 = require("../../types");
const price_1 = require("@melonproject/token-math/price");
const quantity_1 = require("@melonproject/token-math/quantity");
const debug_1 = require("../debug");
const debug = require('debug')('exchange-aggregator:ethfinex');
const subscribeMessage = (options) => {
    const base = options.pair.base.symbol;
    const quote = options.pair.quote.symbol;
    const message = {
        event: 'subscribe',
        channel: 'book',
        symbol: `t${base}${quote}`,
        prec: 'R0',
    };
    return message;
};
const getWebsocketUrl = (options) => {
    switch (options.network) {
        case types_1.Network.MAINNET:
            return 'wss://api.bitfinex.com/ws/2';
        default:
            throw new Error('Ethfinex only supports the MAINNET network.');
    }
};
exports.observeEthfinex = (options) => {
    try {
        const open$ = new Rx.Subject();
        const close$ = new Rx.Subject();
        const url = getWebsocketUrl(options);
        const ws$ = webSocket_1.webSocket({
            WebSocketCtor: isomorphic_ws_1.default,
            closeObserver: close$,
            openObserver: open$,
            url,
        });
        ws$.next(subscribeMessage(options));
        open$.subscribe(() => {
            debug('Opening connection.');
        });
        close$.subscribe(() => {
            debug('Closing connection.');
        });
        const messages$ = ws$.pipe(operators_1.filter(R.is(Array)), operators_1.map(R.nth(1)));
        const snapshots$ = messages$.pipe(operators_1.filter(R.compose(R.is(Array), R.head)), operators_1.map(normalizeSnapshotEvent(options)));
        const updates$ = messages$.pipe(operators_1.filter(R.compose(R.is(Number), R.head)), operators_1.map(normalizeOrderEvent(options)));
        return Rx.merge(snapshots$, updates$).pipe(operators_1.tap(value => {
            debug(...debug_1.debugEvent(value));
        }));
    }
    catch (error) {
        return Rx.throwError(error);
    }
};
const normalizeOrderEvent = R.curryN(2, (options, order) => {
    const [id, price, amount] = order;
    const volume = Math.abs(parseFloat(amount));
    const type = amount > 0 ? types_1.OrderType.BID : types_1.OrderType.ASK;
    // TODO: Figure out the right formula here.
    const trade = price_1.createPrice(quantity_1.createQuantity(options.pair.base, parseFloat(price)), quantity_1.createQuantity(options.pair.quote, volume));
    const event = price === 0 ? types_1.NormalizedMessageType.REMOVE : types_1.NormalizedMessageType.ADD;
    return {
        id: id,
        event,
        exchange: types_1.Exchange.ETHFINEX,
        order: {
            id: id,
            type,
            trade,
            exchange: types_1.Exchange.ETHFINEX,
        },
    };
});
const normalizeSnapshotEvent = R.curryN(2, (options, orders) => {
    const processed = orders
        .filter(([_, price]) => {
        return price !== 0;
    })
        .map(([id, price, amount]) => {
        const type = amount > 0 ? types_1.OrderType.BID : types_1.OrderType.ASK;
        const volume = Math.abs(parseFloat(amount));
        // TODO: Figure out the right formula here.
        const trade = price_1.createPrice(quantity_1.createQuantity(options.pair.base, parseFloat(price)), quantity_1.createQuantity(options.pair.quote, volume));
        return {
            id: id,
            type,
            trade,
            exchange: types_1.Exchange.ETHFINEX,
        };
    });
    return {
        event: types_1.NormalizedMessageType.SNAPSHOT,
        exchange: types_1.Exchange.ETHFINEX,
        orders: processed,
    };
});
