"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Rx = __importStar(require("rxjs"));
const R = __importStar(require("ramda"));
const axios_1 = __importDefault(require("axios"));
const operators_1 = require("rxjs/operators");
const types_1 = require("../../types");
const price_1 = require("@melonproject/token-math/price");
const quantity_1 = require("@melonproject/token-math/quantity");
const debug_1 = require("../debug");
const debug = require('debug')('exchange-aggregator:kraken');
const getHttpUrl = (options) => {
    const base = options.pair.base.symbol;
    const quote = options.pair.quote.symbol;
    switch (options.network) {
        case types_1.Network.MAINNET:
            return `https://api.kraken.com/0/public/Depth?pair=${base}${quote}`;
        default:
            throw new Error('Kraken only supports the MAINNET network.');
    }
};
const normalizeOrder = R.curryN(3, (options, type, order) => {
    // TODO: Figure out the right formula here.
    const price = price_1.createPrice(quantity_1.createQuantity(options.pair.base, parseFloat(order[0])), quantity_1.createQuantity(options.pair.quote, parseFloat(order[1])));
    return {
        type,
        exchange: types_1.Exchange.KRAKEN,
        trade: price,
    };
});
exports.observeKraken = (options) => {
    try {
        const url = getHttpUrl(options);
        const polling$ = Rx.interval(5000).pipe(operators_1.switchMap(() => __awaiter(this, void 0, void 0, function* () {
            const response = yield axios_1.default
                .get(url)
                .then(value => value.data);
            if (response.error && response.error.length) {
                throw new Error(response.error.join(''));
            }
            return response.result;
        })), operators_1.retryWhen(error => error.pipe(operators_1.tap(error => debug(error)), operators_1.delay(10000))), operators_1.map((result) => {
            const key = `X${options.pair.base.symbol}X${options.pair.quote.symbol}`;
            const orderbook = (result && result[key]) || {
                asks: [],
                bids: [],
            };
            return [].concat(orderbook.asks.map(normalizeOrder(options, types_1.OrderType.ASK)), orderbook.bids.map(normalizeOrder(options, types_1.OrderType.BID)));
        }), operators_1.distinctUntilChanged());
        const orders$ = polling$.pipe(operators_1.share());
        return orders$.pipe(operators_1.map((orders) => ({
            event: types_1.NormalizedMessageType.SNAPSHOT,
            exchange: types_1.Exchange.KRAKEN,
            orders,
        })), operators_1.tap(value => {
            debug(...debug_1.debugEvent(value));
        }));
    }
    catch (error) {
        return Rx.throwError(error);
    }
};
