"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const Rx = __importStar(require("rxjs"));
const axios_1 = __importDefault(require("axios"));
const operators_1 = require("rxjs/operators");
const types_1 = require("../../types");
const price_1 = require("@melonproject/token-math/price");
const quantity_1 = require("@melonproject/token-math/quantity");
const debug_1 = require("../debug");
const debug = require('debug')('exchange-aggregator:kyber');
const getCurrenciesHttpUrl = (options) => {
    switch (options.network) {
        case types_1.Network.MAINNET:
            return 'https://api.kyber.network/currencies';
        default:
            throw new Error('Kyber only supports the MAINNET network.');
    }
};
const getRateHttpUrl = (options, type, currency, interval) => {
    if (options.network !== types_1.Network.MAINNET) {
        throw new Error('Kyber only supports the MAINNET network.');
    }
    const prefix = `https://api.kyber.network/${type}_rate`;
    const splits = R.splitEvery(5, interval);
    const suffix = splits
        .map(split => {
        const quantities = split
            .map(qty => {
            return `qty=${qty}`;
        })
            .join('&');
        return `id=${currency}&${quantities}`;
    })
        .join('&');
    return `${prefix}?${suffix}`;
};
const formatResponse = (options, type, response) => {
    if (response.error) {
        throw new Error(`Error trying to fetch Kyber ${type} rates.`);
    }
    const volumeKey = type === types_1.OrderType.ASK ? 'dst_qty' : 'src_qty';
    const priceKey = type === types_1.OrderType.ASK ? 'src_qty' : 'dst_qty';
    const groups = response.data.map(current => {
        return Object.keys(current[volumeKey]).map(index => {
            // TODO: Is this correct?
            const price = price_1.createPrice(quantity_1.createQuantity(options.pair.base, current[priceKey][index]), quantity_1.createQuantity(options.pair.quote, current[volumeKey][index]));
            return {
                type,
                exchange: types_1.Exchange.KYBER,
                trade: price,
            };
        });
    });
    return [].concat(...groups);
};
const pollRate = R.curryN(2, (options, currencies) => {
    try {
        const base = options.pair.base.symbol;
        const quote = options.pair.quote.symbol;
        const currency = currencies.find(R.propEq('symbol', base));
        if (!currency) {
            throw new Error(`The ${base} token is not supported.`);
        }
        // TODO: Pass the interval through configuration.
        const interval = [1, 10, 20, 30, 40, 50, 75, 100, 125, 150];
        const buyUrl = getRateHttpUrl(options, 'buy', currency.id, interval);
        const sellUrl = getRateHttpUrl(options, 'sell', currency.id, interval);
        return Rx.interval(5000).pipe(operators_1.tap(() => {
            debug(`Loading snapshot for market %s-%s.`, base, quote);
        }), operators_1.switchMap(() => __awaiter(this, void 0, void 0, function* () {
            const sellRequest = axios_1.default
                .get(sellUrl)
                .then(result => result.data);
            const buyRequest = axios_1.default
                .get(buyUrl)
                .then(result => result.data);
            const [buyResponse, sellResponse] = yield Promise.all([
                buyRequest,
                sellRequest,
            ]);
            return [buyResponse, sellResponse];
        })), operators_1.retryWhen(error => error.pipe(operators_1.tap(error => debug(error)), operators_1.delay(10000))), operators_1.distinctUntilChanged(), operators_1.map(([buyResponse, sellResponse]) => [].concat(formatResponse(options, types_1.OrderType.BID, buyResponse), formatResponse(options, types_1.OrderType.ASK, sellResponse))));
    }
    catch (error) {
        return Rx.throwError(error);
    }
});
exports.observeKyber = (options) => {
    try {
        if (options.pair.quote.symbol !== 'ETH') {
            throw new Error('Kyber only support ETH as a quote token.');
        }
        const url = getCurrenciesHttpUrl(options);
        const currencies$ = Rx.defer(() => __awaiter(this, void 0, void 0, function* () {
            const response = yield axios_1.default
                .get(url)
                .then(result => result.data);
            if (response.error) {
                throw new Error(response.error);
            }
            return response.data;
        })).pipe(operators_1.retryWhen(error => error.pipe(operators_1.tap(error => debug(error)), operators_1.delay(10000))));
        const rates$ = currencies$.pipe(operators_1.switchMap(pollRate(options)));
        return rates$.pipe(operators_1.map((orders) => ({
            event: types_1.NormalizedMessageType.SNAPSHOT,
            exchange: types_1.Exchange.KYBER,
            orders,
        })), operators_1.tap(value => {
            debug(...debug_1.debugEvent(value));
        }));
    }
    catch (error) {
        return Rx.throwError(error);
    }
};
